// src/use-switch.ts
import { useCallback, useId, useRef } from "react";
import { mapPropsVariants } from "@nextui-org/system";
import { useHover } from "@react-aria/interactions";
import { toggle } from "@nextui-org/theme";
import { chain, mergeProps } from "@react-aria/utils";
import { clsx, dataAttr } from "@nextui-org/shared-utils";
import { useFocusableRef } from "@nextui-org/dom-utils";
import { useSwitch as useReactAriaSwitch } from "@react-aria/switch";
import { useMemo } from "react";
import { useToggleState } from "@react-stately/toggle";
import { useFocusRing } from "@react-aria/focus";
function useSwitch(originalProps) {
  const [props, variantProps] = mapPropsVariants(originalProps, toggle.variantKeys);
  const {
    ref,
    as,
    name,
    value = "",
    isReadOnly = false,
    autoFocus = false,
    leftIcon,
    rightIcon,
    defaultSelected,
    isSelected: isSelectedProp,
    children,
    thumbIcon,
    className,
    styles,
    onChange,
    onValueChange,
    ...otherProps
  } = props;
  const Component = as || "label";
  const inputRef = useRef(null);
  const domRef = useFocusableRef(ref, inputRef);
  const labelId = useId();
  const ariaSwitchProps = useMemo(() => {
    const ariaLabel = otherProps["aria-label"] || typeof children === "string" ? children : void 0;
    return {
      name,
      value,
      children,
      autoFocus,
      defaultSelected,
      isSelected: isSelectedProp,
      isDisabled: !!originalProps.isDisabled,
      isReadOnly,
      "aria-label": ariaLabel,
      "aria-labelledby": otherProps["aria-labelledby"] || labelId,
      onChange: onValueChange
    };
  }, [
    value,
    name,
    labelId,
    children,
    autoFocus,
    isSelectedProp,
    defaultSelected,
    originalProps.isDisabled,
    otherProps["aria-label"],
    otherProps["aria-labelledby"],
    onValueChange
  ]);
  const state = useToggleState(ariaSwitchProps);
  const { inputProps } = useReactAriaSwitch(ariaSwitchProps, state, inputRef);
  const { focusProps, isFocused, isFocusVisible } = useFocusRing({ autoFocus: inputProps.autoFocus });
  const { hoverProps, isHovered } = useHover({
    isDisabled: inputProps.disabled
  });
  const isSelected = inputProps.checked;
  const isDisabled = inputProps.disabled;
  const slots = useMemo(
    () => toggle({
      ...variantProps,
      isFocusVisible
    }),
    [...Object.values(variantProps), isFocusVisible]
  );
  const baseStyles = clsx(styles == null ? void 0 : styles.base, className);
  const getBaseProps = () => {
    return {
      ref: domRef,
      className: slots.base({ class: baseStyles }),
      "data-disabled": dataAttr(isDisabled),
      "data-checked": dataAttr(isSelected),
      ...mergeProps(hoverProps, otherProps)
    };
  };
  const getWrapperProps = useCallback(() => {
    return {
      "data-hover": dataAttr(isHovered),
      "data-checked": dataAttr(isSelected),
      "data-focus": dataAttr(isFocused),
      "data-focus-visible": dataAttr(isFocused && isFocusVisible),
      "data-disabled": dataAttr(isDisabled),
      "data-readonly": dataAttr(inputProps.readOnly),
      "aria-hidden": true,
      className: clsx(slots.wrapper({ class: styles == null ? void 0 : styles.wrapper }))
    };
  }, [
    slots,
    styles == null ? void 0 : styles.wrapper,
    isHovered,
    isSelected,
    isFocused,
    isFocusVisible,
    isDisabled,
    inputProps.readOnly
  ]);
  const getInputProps = () => {
    return {
      ref: inputRef,
      id: inputProps.id,
      ...mergeProps(inputProps, focusProps),
      onChange: chain(onChange, inputProps.onChange)
    };
  };
  const getThumbProps = useCallback(
    () => ({
      "data-checked": dataAttr(isSelected),
      "data-focus": dataAttr(isFocused),
      "data-focus-visible": dataAttr(isFocused && isFocusVisible),
      "data-disabled": dataAttr(isDisabled),
      "data-readonly": dataAttr(inputProps.readOnly),
      className: slots.thumb({ class: styles == null ? void 0 : styles.thumb })
    }),
    [slots, styles == null ? void 0 : styles.thumb, isSelected, isFocused, isFocusVisible, isDisabled, inputProps.readOnly]
  );
  const getLabelProps = useCallback(
    () => ({
      id: labelId,
      "data-disabled": dataAttr(isDisabled),
      "data-checked": dataAttr(isSelected),
      className: slots.label({ class: styles == null ? void 0 : styles.label })
    }),
    [slots, styles == null ? void 0 : styles.label, isDisabled, isSelected]
  );
  const getThumbIconProps = useCallback(
    (props2 = {
      includeStateProps: false
    }) => mergeProps(
      {
        width: "1em",
        height: "1em",
        "data-checked": dataAttr(isSelected),
        className: slots.thumbIcon({ class: styles == null ? void 0 : styles.thumbIcon })
      },
      props2.includeStateProps ? {
        isSelected
      } : {}
    ),
    [slots, styles == null ? void 0 : styles.thumbIcon, isSelected, originalProps.disableAnimation]
  );
  const getLeftIconProps = useCallback(
    () => ({
      width: "1em",
      height: "1em",
      "data-checked": dataAttr(isSelected),
      className: slots.leftIcon({ class: styles == null ? void 0 : styles.leftIcon })
    }),
    [slots, styles == null ? void 0 : styles.leftIcon, isSelected]
  );
  const getRightIconProps = useCallback(
    () => ({
      width: "1em",
      height: "1em",
      "data-checked": dataAttr(isSelected),
      className: slots.rightIcon({ class: styles == null ? void 0 : styles.rightIcon })
    }),
    [slots, styles == null ? void 0 : styles.rightIcon, isSelected]
  );
  return {
    Component,
    slots,
    styles,
    domRef,
    children,
    thumbIcon,
    leftIcon,
    rightIcon,
    isHovered,
    isSelected,
    isFocused,
    isFocusVisible,
    isDisabled,
    getBaseProps,
    getWrapperProps,
    getInputProps,
    getLabelProps,
    getThumbProps,
    getThumbIconProps,
    getLeftIconProps,
    getRightIconProps
  };
}

export {
  useSwitch
};
